package appchain

import (
	"context"

	"github.com/sirupsen/logrus"

	"gitlab.33.cn/link33/sidecar/internal/port"
	"gitlab.33.cn/link33/sidecar/internal/txcrypto"
	"gitlab.33.cn/link33/sidecar/pkg/plugins"
	"gitlab.33.cn/link33/sidecar/tool"
)

type Manager struct {
	ctx       context.Context
	cancel    context.CancelFunc
	g         *tool.Group
	appChains []*appChain
}

func NewManager(clients []plugins.Client, cryptor txcrypto.Cryptor, logger logrus.FieldLogger) *Manager {
	ctx, cancel := context.WithCancel(context.Background())
	g, _ := tool.WithContext(ctx)
	return &Manager{
		ctx:       ctx,
		cancel:    cancel,
		g:         g,
		appChains: newAppchains(ctx, clients, cryptor, logger),
	}
}

func (m *Manager) Start() error {
	for _, appChain := range m.appChains {
		a := appChain
		m.g.Go(func() error {
			return a.Start()
		})
	}
	return m.g.Wait()
}

func (m *Manager) Stop() error {
	for _, appChain := range m.appChains {
		a := appChain
		m.g.Go(func() error {
			return a.Stop()
		})
	}
	m.cancel()
	return m.g.Wait()
}

func (m *Manager) Ports() []port.Port {
	var ports []port.Port
	for _, appChain := range m.appChains {
		ports = append(ports, appChain)
	}
	return ports
}
